/***************************************************************************
                          krpmbuilder.cpp  -  description
                             -------------------
    begin                : Don Aug  1 15:56:02 CEST 2002
    copyright            : (C) 2002 by Olaf Lueg
                           (C) 2005 by Wolfgang Beutner
    email                : olueg@olsd.de
                           wolfgang.beutner(at)gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>
#include <sstream>
#include "krpmbuilder.h"
#include "projectpage.h"
#include "filespage.h"
#include "packagepage.h"
#include "buildpage.h"
#include "startpage.h"
#include "finishpage.h"
#include "setting/settings.h"
#include "setting/paths.h"

// qt include
#include <qlabel.h>
#include <qpushbutton.h>
#include <qlistview.h>
#include <qheader.h>
#include <qfile.h>
#include <qtextbrowser.h>
#include <qregexp.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qdir.h>

// kde include
#include <kfiledialog.h>
#include <klocale.h>
#include <kurl.h>
#include <kaboutapplication.h>
#include <kstddirs.h>
#include <klineedit.h>
#include <kcombobox.h>
#include <kiconloader.h>
#include <kpushbutton.h>
#include <kmessagebox.h>

KRPMBuilder::KRPMBuilder(QWidget *parent, const char *name) : KWizard(parent, name)
{
  this->setMinimumSize(QSize( 940, 480 ) );
  hasPaths = false;
  saveProject = false;
  projectPage = new ProjectPage(this);
  filesPage = new FilesPage(this);
  filesPage->hide();
  packagePage = new PackagePage(this);
  buildPage = new BuildPage(this);
  finishPage = new FinishPage(this);
  KStandardDirs dir;
  projectPage->fileOpen->setPixmap(KGlobal::instance()->iconLoader()->loadIcon( "fileopen" ,KIcon::NoGroup ,     KIcon::SizeSmall )  );
  projectPage->fileSave->setPixmap(KGlobal::instance()->iconLoader()->loadIcon( "filesave" ,KIcon::NoGroup , KIcon::SizeSmall )  );
  projectPage->fileNew->setPixmap(KGlobal::instance()->iconLoader()->loadIcon( "filenew" ,KIcon::NoGroup , KIcon::SizeSmall )  );
  projectPage->fileSettings->setPixmap(KGlobal::instance()->iconLoader()->loadIcon( "configure" ,KIcon::NoGroup , KIcon::SizeSmall )  );
  pLogo =  QPixmap(dir.findResource("data","krpmbuilder/pics/logo.png") );
  pSpacer =  QPixmap(dir.findResource("data","krpmbuilder/pics/fill.png") );
  projectPage->p_logo->setPixmap(pLogo);
  projectPage->p_spacer->setPixmap(pSpacer);
  filesPage->p_logo->setPixmap(pLogo);
  filesPage->p_spacer->setPixmap(pSpacer);
  packagePage->p_logo->setPixmap(pLogo);
  packagePage->p_spacer->setPixmap(pSpacer);
  buildPage->p_logo->setPixmap(pLogo);
  buildPage->p_spacer->setPixmap(pSpacer);
  finishPage->p_logo->setPixmap(pLogo);
  finishPage->p_spacer->setPixmap(pSpacer);

  addPage(projectPage,"Step One: General Settings");
//  addPage(filesPage,i18n("Step two: Select additional sources and patches"));
  addPage(packagePage,i18n("Step three: more information for the package"));
  addPage(buildPage,i18n("Step four: build options"));
  addPage(finishPage,i18n("Step five: build the package"));
  setNextEnabled(projectPage,true);
//  startPage = new StartPage(0);
//  startPage->show();
  readConfig();
//	wizard->setNextEnabled(pageRpmBuild,false);
//	wizard->setTitle(infoPage,"Step one: Welcome to KRPMBuilder Wizard");

  about = new KAboutApplication(this);
  helpButton()->setText(i18n("&About"));
  cancelButton()->setText(i18n("&Exit"));
  finishPage->stop->setEnabled(false);
  filesPage->FileList->clear();
  filesPage->PatchList->clear();

  connect((QObject*)projectPage->fileOpen,SIGNAL(clicked()),this,SLOT(slotProjectOpen()));
  connect(cancelButton(),SIGNAL(clicked()),this,SLOT(slotQuit()));
  connect((QObject*)finishPage->create,SIGNAL(clicked()),this,SLOT(slotCreateRPM()));
  connect((QObject*)finishPage->stop,SIGNAL(clicked()),this,SLOT(slotStopRPM()));
  connect(this,SIGNAL(helpClicked()),this,SLOT(slotAbout()));
  connect((QObject*)projectPage->fileNew,SIGNAL(clicked()),this,SLOT(slotNew()));
  connect((QObject*)projectPage->fileSave,SIGNAL(clicked()),this,SLOT(slotSave()));
  connect(filesPage->FileList,SIGNAL(highlighted(int)),this,SLOT(slotFileHighlighted( int )));
  connect(filesPage->PatchList,SIGNAL(highlighted(int)),this,SLOT(slotPatchHighlighted( int )));
  connect((QObject*)filesPage->fileAdd,SIGNAL(clicked()),this,SLOT(slotFileAdd()));
  connect((QObject*)filesPage->fileDelete,SIGNAL(clicked()),this,SLOT(slotFileDelete()));
  connect((QObject*)filesPage->moveUp,SIGNAL(clicked()),this,SLOT(slotFileMoveUp()));
  connect((QObject*)filesPage->moveDown,SIGNAL(clicked()),this,SLOT(slotFileMoveDown()));
  connect((QObject*)filesPage->patchAdd,SIGNAL(clicked()),this,SLOT(slotPatchAdd()));
  connect((QObject*)filesPage->patchDelete,SIGNAL(clicked()),this,SLOT(slotPatchDelete()));
  connect((QObject*)filesPage->patchUp,SIGNAL(clicked()),this,SLOT(slotPatchMoveUp()));
  connect((QObject*)filesPage->patchDown,SIGNAL(clicked()),this,SLOT(slotPatchMoveDown()));
  connect((QObject*)filesPage->patchSearch,SIGNAL(clicked()),this,SLOT(slotPatchSearch()));
  connect((QObject*)filesPage->fileSearch,SIGNAL(clicked()),this,SLOT(slotFileSearch()));
  connect((QObject*)projectPage->fileSearch,SIGNAL(clicked()),this,SLOT(slotSourceSearch()));
  connect((QObject*)projectPage->fileSettings,SIGNAL(clicked()),this,SLOT(slotSettings()));
}

KRPMBuilder::~KRPMBuilder()
{
}
/** No descriptions */
void KRPMBuilder::slotProjectOpen()
{
  QString projectFile( KFileDialog::getOpenFileName( QString::null,"*.krpm",this, i18n("Open...") ) );
  if ( projectFile.isEmpty() )
    return;
  QFile f( projectFile );
  if ( f.open( IO_ReadOnly ) ) {
    f.close();
  } else {
    KMessageBox::error( 0 , //No parent
      i18n("Open failed") , //Short!
      i18n("Error while reading") ); //Caption
  return;
  }
  KConfig *config = new KConfig(projectFile );
//  debug(projectFile);
  config->setGroup("General");
  QString m_version = config->readEntry("App Version","0");
  if ( m_version == "0" ){
    KMessageBox::sorry(this,i18n("This is an older saved file!\nNot supported in this version!"),i18n("Sorry"));
    return;
  }
// projectPage
  projectPage->m_name->setText(config->readEntry("Name","") );
  projectPage->m_version->setText(config->readEntry("Version", "") ) ;
  projectPage->m_release->setText(config->readEntry("Release","") );
  projectPage->m_url->setText(config->readEntry("URL","") );
  projectPage->m_packager->setText(config->readEntry("Packager","") ) ;
  if ( findItem(projectPage->c_copyright, config->readEntry("Copyright","GPL") ) ){
    projectPage->c_copyright->setCurrentText(config->readEntry("Copyright","GPL") );
  }  else  {
    projectPage->c_copyright->insertItem(config->readEntry("Copyright","GPL") );
    projectPage->c_copyright->setCurrentText(config->readEntry("Copyright","GPL") );
  }
  projectPage->m_file->setText(config->readEntry("Source file","%{name}-%{version}.tar.gz"));
  sourceDir.setPath( config->readEntry("Source path", "" ) );
// files Page
  filesPage->FileList->clear();
  filesPage->PatchList->clear();
  filesPage->FileList->insertStringList(config->readListEntry("Files") );
  filesPage->PatchList->insertStringList(config->readListEntry("Patches") );
  filesPage->fileSearch->setEnabled( true );
  filesPage->patchSearch->setEnabled( true );
// package Page
  packagePage->m_distribution->setText(config->readEntry("Distribution","") );
  if ( findItem(packagePage->c_group, config->readEntry("Group","X11") ) ) {
    packagePage->c_group->setCurrentText(config->readEntry("Group","X11") );
  } else {
    packagePage->c_group->insertItem(config->readEntry("Group","X11") );
    packagePage->c_group->setCurrentText(config->readEntry("Group","X11") );
  }
  packagePage->m_provides->setText(config->readEntry("Provides","") );
  packagePage->m_requires->setText(config->readEntry("Requires","") );
  packagePage->m_vendor->setText(config->readEntry("Vendor","") );
  packagePage->m_summary->setText(config->readEntry("Summary","") );
  packagePage->m_description->setText(config->readEntry("Description","") );
// build Page
  buildPage->m_buildRoot->setText(config->readEntry("BuildRoot","%{_tmppath}/build-root-%{name}") );
  buildPage->m_prefix->setText(config->readEntry("Prefix","") );
  buildPage->m_configure->setText(config->readEntry("Configure","") );
  if ( findItem(buildPage->c_processor, config->readEntry("Processor","i386") ) ) {
    buildPage->c_processor->setCurrentText(config->readEntry("Processor","i386") );
  } else {
    buildPage->c_processor->insertItem(config->readEntry("Processor","i386") );
    buildPage->c_processor->setCurrentText(config->readEntry("Processor","i386") );
  }
  buildPage->c_binary->setChecked(config->readBoolEntry("Build binary",false) );
  buildPage->c_source->setChecked(config->readBoolEntry("Build source",true) );
  delete config;
  // Config is read now. We need to move selected files to
  // the rpm SOURCE directory to insure they are in place before
  // we start the build. Nothing has to be displayed.
  copyAllFilesToSOURCE();
  saveProject = false;
}

/** No descriptions */
void KRPMBuilder::slotAbout()
{
  about->show();
}
/** No descriptions */
void KRPMBuilder::slotQuit()
{
  saveConfig();
  // Something of the project has been changed display a messagebox and
  // ask the user if he wants to save it.
  if ( saveProject || projectIsModified() ) {
    if ( KMessageBox::questionYesNo( this, i18n( "Project has been changed, save it?" ) ) == KMessageBox::Yes ) {
      slotSave();
    }
  }  
  close();
}
/** Standrd mode - only one source file, and no patches */
void KRPMBuilder::slotCreateRPM()
{
  if ( ! hasPaths ){
    KMessageBox::information(this,i18n("Please set the path to the RPM SOURCE and SPECS directory!") );
    return slotSettings();
  }
//update Combo boxes
  if ( !findItem(projectPage->c_copyright,projectPage->c_copyright->currentText()) ) {
    projectPage->c_copyright->insertItem(projectPage->c_copyright->currentText() );
  }
  if (!findItem(packagePage->c_group,packagePage->c_group->currentText()) ){
    packagePage->c_group->insertItem(packagePage->c_group->currentText() );
  }
  if (!findItem(buildPage->c_processor,buildPage->c_processor->currentText()) ){
    buildPage->c_processor->insertItem(buildPage->c_processor->currentText() );
  }
  if ( hasEmptyTags() ) {
    return;
  }
  finishPage->Output->setText(i18n("Building the spec file...") );
  KStandardDirs dir;
  QFile f(dir.findResource("data","krpmbuilder/specs/standard.spec") );
  if ( !f.open(IO_ReadOnly) ) {
    finishPage->Output->setText(finishPage->Output->text() +"    " + i18n("Error")+"\n"+i18n("The file   standard.spec is not fount in:" ) +"\n" +dir.findResource("data","krpmbuilder/specs/") ) ;
    return;
  }
  QTextStream t(&f);
  QString s = t.read();
  f.close();
  //std::cout << s << std::endl;
  //parse project page
  s.replace( QRegExp( "\\@version\\@" ), projectPage->m_version->text() );
//  s.replace( QRegExp( "\\@name\\@" ), projectPage->m_name->text().lower() );
  s.replace( QRegExp( "\\@name\\@" ), projectPage->m_name->text() );
  s.replace( QRegExp( "\\@release\\@" ), projectPage->m_release->text() );
  s.replace( QRegExp( "\\@url\\@" ), projectPage->m_url->text() );
  s.replace( QRegExp( "\\@packager\\@" ), projectPage->m_packager->text() );
  s.replace( QRegExp( "\\@license\\@" ), projectPage->c_copyright->currentText() );
  s.replace( QRegExp( "\\@source\\@" ), projectPage->m_file->text() );
  // parse files page
  QString additionalFiles = "";
  QString additionalSetup = "";
  if ( filesPage->FileList->count() > 0 ) {
    // Additional files were entered. Set up the paths and modify setup macro
    for ( uint i = 1; i <= filesPage->FileList->count(); i++ ) {
      std::ostringstream str;
      str << i;
      if ( i > 1 ) {
        additionalFiles += '\n';
      }
      additionalSetup += " -b " + str.str();
      additionalFiles += "Source" + str.str() + ":" + filesPage->FileList->text( i - 1 );
    }
  }
  s.replace( QRegExp( "\\@moresrc\\@" ), additionalSetup );
  s.replace( QRegExp( "\\@additional\\@" ), additionalFiles );
  //parse package page
  s.replace( QRegExp( "\\@distribution\\@" ), packagePage->m_distribution->text() );
  if (  packagePage->m_provides->text().isEmpty() ){
    s.replace( QRegExp( "\\@provides\\@" ), packagePage->m_provides->text() );
  } else {
    s.replace( QRegExp( "\\@provides\\@" ), "Provides: "+packagePage->m_provides->text() );
  }
  if (packagePage->m_requires->text().isEmpty() ){
    s.replace( QRegExp( "\\@requires\\@" ), packagePage->m_requires->text() );
  } else {
    s.replace( QRegExp( "\\@requires\\@" ), "Requires: "+packagePage->m_requires->text() );
  }
  if ( packagePage->m_vendor->text().isEmpty() ){
    s.replace( QRegExp( "\\@vendor\\@" ), packagePage->m_vendor->text() );
  } else {
    s.replace( QRegExp( "\\@vendor\\@" ), "Vendor: "+packagePage->m_vendor->text() );
  }
  s.replace( QRegExp( "\\@summary\\@" ), packagePage->m_summary->text() );
  s.replace( QRegExp( "\\@description\\@" ), packagePage->m_description->text() );
  s.replace( QRegExp( "\\@group\\@" ), packagePage->c_group->currentText() );
  // parse build page
  s.replace( QRegExp( "\\@build-root\\@" ), buildPage->m_buildRoot->text() );
  s.replace( QRegExp( "\\@prefix\\@" ), buildPage->m_prefix->text() );
  s.replace( QRegExp( "\\@configure\\@" ), buildPage->m_configure->text() );
  QString processor = buildPage->c_processor->currentText();
  bool buildBinary =buildPage->c_binary->isChecked();
  bool buildSource = buildPage->c_source->isChecked();
  QString filename;
  filename = projectPage->m_name->text().lower() + "-" + projectPage->m_version->text() +"-" +   projectPage->m_release->text() + ".spec" ;
  QFile fOut;
  QDir::setCurrent( RPMspecURL.path( true ) );
  fOut.setName(filename);
  if ( fOut.exists() ) {
    fOut.remove();
  }
  if ( ! fOut.open( IO_ReadWrite ) ) {
    finishPage->Output->setText(finishPage->Output->text() +"\n"+i18n("Couldn't create spec file!") +"\n" + i18n("Do you have write acces?") );	
    fOut.close();
    return;
  }
  QTextStream tOut(&fOut);
  tOut << s;
  fOut.close();
  finishPage->Output->setText(finishPage->Output->text() + "    " + i18n("done") + "\n" );
  finishPage->Output->setText(finishPage->Output->text() +i18n("Building package now...") +"\n");
  finishPage->create->setEnabled(false);
  finishPage->stop->setEnabled(true);
  runRPM(buildBinary,buildSource,processor,RPMspecURL.path(true) +filename);
}
/** No descriptions */
void KRPMBuilder::saveConfig(){
  if ( ! findItem(projectPage->c_copyright,projectPage->c_copyright->currentText()) ){
    projectPage->c_copyright->insertItem(projectPage->c_copyright->currentText() );
  }
  if ( ! findItem(packagePage->c_group,packagePage->c_group->currentText()) ){
    packagePage->c_group->insertItem(packagePage->c_group->currentText() );
  }
  if ( !findItem(buildPage->c_processor,buildPage->c_processor->currentText()) ){
    buildPage->c_processor->insertItem(buildPage->c_processor->currentText() );
  }
  KConfig *config = KGlobal::config();
  config->setGroup("General");
  config->writeEntry("SOURCE",RPMsourceURL.path(true) );
  config->writeEntry("SPECS",RPMspecURL.path(true) );
  QStringList list;
  for(int i = 0; i < projectPage->c_copyright->count() ; i++){
    list.append(projectPage->c_copyright->text(i) );
  }
  config->writeEntry("Copyright list",list);
  list.clear();
  for(int i = 0; i < buildPage->c_processor->count() ; i++){
    list.append(buildPage->c_processor->text(i) );
  }
  config->writeEntry("Processor list",list);
  list.clear();
  for(int i = 0; i < packagePage->c_group->count() ; i++){
    list.append(packagePage->c_group->text(i) );
  }
  config->writeEntry("Group list",list);
}
/** No descriptions */
void KRPMBuilder::readConfig(){
  KConfig *config = KGlobal::config();
  QStringList list;
  config->setGroup("General");
  RPMsourceURL = config->readEntry("SOURCE",QString::null);
  RPMspecURL =config->readEntry("SPECS",QString::null);
  if ( RPMsourceURL.path() == "" || RPMspecURL.path() == "" ) {
    KMessageBox::information(this,i18n("Please set the path to the RPM SOURCE and SPECS directory!") );
    slotSettings();
  } else {
    hasPaths = true;
  }
  list = config->readListEntry("Copyright list");
  if ( list.count() != 0 ){
    projectPage->c_copyright->clear();
    projectPage->c_copyright->insertStringList(list);
  }
  list = config->readListEntry("Processor list");
  if ( list.count() != 0){
    buildPage->c_processor->clear();
    buildPage->c_processor->insertStringList(list);
  }
  list = config->readListEntry("Group list");
  if ( list.count() != 0){
    packagePage->c_group->clear();
    packagePage->c_group->insertStringList(list);
  }
}
/** No descriptions */
void KRPMBuilder::runRPM(bool buildBinary,bool buildSource,QString processor,QString filename){
  proc = new KShellProcess();
  if ( buildSource ){
    (*proc) << "rpmbuild" << "-ba" << "--target" << processor << filename;
  }
  if ( buildBinary ) {
    (*proc) << "rpmbuild" << "-bb" << "--target" << processor << filename;
  }
  connect(proc,SIGNAL(receivedStderr(KProcess *,char *,int)),this,SLOT(slotUpdateInfo(KProcess *,char *,int)));
  connect(proc,SIGNAL(receivedStdout(KProcess *,char *,int)),this,SLOT(slotUpdateInfo(KProcess *,char *,int)));
  connect(proc,SIGNAL(processExited(KProcess*)),this,SLOT(slotRpmDone(KProcess*)));
  //start the process
  proc->start(KProcess::NotifyOnExit,KProcess::All);

}
/** No descriptions */
void KRPMBuilder::slotUpdateInfo(KProcess*,char *buf, int len){
  QString info = buf;
  info = info.left(len);
  finishPage->Output->setText(finishPage->Output->text() + info);
  finishPage->Output->scrollToBottom();
}
/** No descriptions */
void KRPMBuilder::slotRpmDone(KProcess*){
  finishPage->create->setEnabled(true);
  finishPage->stop->setEnabled(false);
}
/** No descriptions */
void KRPMBuilder::slotNew(){
// clear data
  sourceDir.setPath( "" );
  saveProject = false;
// clear all fields
// project page
  projectPage->m_name->setText("");
  projectPage->m_version->setText("");
  projectPage->m_release->setText("");
  projectPage->m_url->setText("http://");
  projectPage->m_packager->setText("");
  projectPage->m_file->setText("%{name}-%{version}.tar.gz");
  //filesPage
  filesPage->FileList->clear();
  filesPage->PatchList->clear();
  filesPage->fileSearch->setEnabled( false );
  filesPage->patchSearch->setEnabled( false );
  
        // files page
/*
	filesPage->m_file->setText("%{name}-%{version}.tar.gz");
	filesPage->FileList->insertItem("%{name}-%{version}.tar.gz");

	filesPage->m_patch->setText("");
*/
	// package page
  packagePage->m_distribution->setText("");
  packagePage->m_provides->setText("");
  packagePage->m_vendor->setText("");
  packagePage->m_requires->setText("");
  packagePage->m_summary->setText("");
  packagePage->m_description->setText("");
// build Page
  buildPage->m_buildRoot->setText("%{_tmppath}/build-root-%{name}");
  buildPage->m_prefix->setText("/opt/kde3");
  buildPage->m_configure->setText("--prefix=%{prefix}");
  buildPage->c_source->setChecked(true);
// finish Page
  finishPage->Output->setText("");
}
/** No descriptions */
void KRPMBuilder::slotSave(){
  QString projectFile( KFileDialog::getSaveFileName( QString::null,"*.krpm",this, i18n("Save as...") ) );
  if ( projectFile.isEmpty() )return;
  if( ! projectFile.contains(".krpm",false)){
    projectFile += ".krpm";
  }
  QFile f( projectFile );
  if ( f.open( IO_WriteOnly ) )
  {
    f.close();
  } else {
    KMessageBox::error( 0 , //No parent
      i18n("Save failed") , //Short!
      i18n("Error while saving") ); //Caption
      return;
  }
  KConfig *config = new KConfig(projectFile );
//  debug(projectFile);
  config->setGroup("General");
  config->writeEntry("App Version","1.0");
// projectPage
  config->writeEntry("Name",projectPage->m_name->text());
  config->writeEntry("Version",projectPage->m_version->text() ) ;
  config->writeEntry("Release",projectPage->m_release->text() );
  config->writeEntry("URL",projectPage->m_url->text() );
  config->writeEntry("Packager",projectPage->m_packager->text() ) ;
  config->writeEntry("Copyright",projectPage->c_copyright->currentText() );
  config->writeEntry("Source file",projectPage->m_file->text() );
  config->writeEntry("Source path", sourceDir.path( -1 ) );
// files Page
  QStringList list;
  // Write list of files to file
  for( uint i =0; i < filesPage->FileList->count(); i++ ){
    list.append(filesPage->FileList->text( i ) );
  }
  config->writeEntry("Files",list);
  list.clear();
  //write list of patches to file
  for( uint i =0; i < filesPage->PatchList->count(); i++ ){
    list.append(filesPage->PatchList->text( i ) );
  }
  config->writeEntry("Patches",list);

// package Page
  config->writeEntry("Distribution",packagePage->m_distribution->text() );
  config->writeEntry("Group",packagePage->c_group->currentText() );
  config->writeEntry("Provides",packagePage->m_provides->text() );
  config->writeEntry("Requires",packagePage->m_requires->text() );
  config->writeEntry("Vendor",packagePage->m_vendor->text() );
  config->writeEntry("Summary",packagePage->m_summary->text() );
  config->writeEntry("Description",packagePage->m_description->text() );
// build Page
  config->writeEntry("BuildRoot",buildPage->m_buildRoot->text() );
  config->writeEntry("Prefix",buildPage->m_prefix->text() );
  config->writeEntry("Configure",buildPage->m_configure->text() );
  config->writeEntry("Processor",buildPage->c_processor->currentText() );
  config->writeEntry("Build binary",buildPage->c_binary->isChecked() );
  config->writeEntry("Build source",buildPage->c_source->isChecked() );
  config->sync();
  delete config;
  saveProject = false;
}
/** No descriptions */
bool KRPMBuilder::findItem(KComboBox *combo,const QString &item) {
  for( int i=0; i < combo->count(); i++ ) {
    if ( combo->text(i) == item ){
      return true;
    }
  }
  return false;
}
/** No descriptions */
bool KRPMBuilder::hasEmptyTags(){
  bool ok = false;
  finishPage->Output->setText(i18n("Checking your settings...")+"\n");
  if ( projectPage->m_name->text() == ""){
    ok = true;
    finishPage->Output->setText(finishPage->Output->text() + i18n("Projectname is missing!!!")+"\n");
  }
  if( projectPage->m_version->text() == ""){
    ok = true;
    finishPage->Output->setText(finishPage->Output->text() + i18n("Project version is missing!!!")+"\n");
  }
  if ( projectPage->m_release->text() == ""){
    ok = true;
    finishPage->Output->setText( finishPage->Output->text() + i18n("Release number is missing!!!")+"\n");
  }
  if ( projectPage->m_url->text() == ""){
    ok = true;
    finishPage->Output->setText( finishPage->Output->text() + i18n("Homepage is missing!!!")+"\n");
   }
  if ( projectPage->m_packager->text() == ""){
    ok = true;
    finishPage->Output->setText( finishPage->Output->text() + i18n("Packager name is missing!!!")+"\n");
  }
  if ( projectPage->m_file->text() == ""){
    ok = true;
    finishPage->Output->setText( finishPage->Output->text() + i18n("Source file is missing!!!")+"\n");
  }

// We do not check anything in the files page
// since we do not need additional files or patches
// package page
  if ( packagePage->m_distribution->text() == ""){
    ok = true;
    finishPage->Output->setText( finishPage->Output->text() + i18n("Distribution is missing!!!")+"\n");
  }
  if ( packagePage->m_summary->text() == "" ){
    ok = true;
    finishPage->Output->setText( finishPage->Output->text() + i18n("Summary is missing!!!")+"\n");
  }
  if ( packagePage->m_description->text() == ""){
    ok = true;
    finishPage->Output->setText( finishPage->Output->text() + i18n("Description is missing!!!")+"\n");
  }
  // build Page
  if ( buildPage->m_buildRoot->text() == "" ){
    ok = true;
    finishPage->Output->setText( finishPage->Output->text() + i18n("Build-Root is missing!!!")+"\n");
  }
  if ( ! ok ) {
    finishPage->Output->setText( finishPage->Output->text() + i18n("Settings OK")+"\n");
  }
  finishPage->Output->scrollToBottom();
  return ok;
}
/** No descriptions */
void KRPMBuilder::slotFileHighlighted(int index){
  filesPage->fileDelete->setEnabled(true);
  enableUpDownButtons( index, (int) filesPage->FileList->count(), filesPage->moveUp, filesPage->moveDown );
}

/** No descriptions */
void KRPMBuilder::slotPatchHighlighted(int index){
  filesPage->patchDelete->setEnabled( true );
  enableUpDownButtons( index, (int) filesPage->PatchList->count(), filesPage->patchUp, filesPage->patchDown );
}

void KRPMBuilder::enableUpDownButtons( const int index, const int itemCount, KPushButton *upButton, KPushButton *downButton )
{  
  if ( index == -1 || index == 0 && itemCount == 1 ) {
    upButton->setEnabled( false );
    downButton->setEnabled( false );
    return;
  }
  if ( index != itemCount - 1 ) {
    upButton->setEnabled( true );
    downButton->setEnabled( true );
  }
  if ( index == itemCount - 1 && index != 0 ) {
    upButton->setEnabled( true );
    downButton->setEnabled( false );
  }
  if ( index == 0 ){
    upButton->setEnabled( false );
  }
}

/** No descriptions */
void KRPMBuilder::slotFileAdd() {
  if ( filesPage->m_file->text() != "" ) {
    // The file gets copied to rpm-SOURCE directory.
    filesPage->m_file->clear();
    filesPage->fileAdd->setEnabled( false );
    targetWidget = filesPage->FileList;
    copyFileToSOURCE( sourceURL );
    saveProject = true;
  }
}

/** No descriptions */
void KRPMBuilder::slotFileDelete(){
  QString Filename = KURL( filesPage->FileList->currentText() ).fileName( true );
  filesPage->FileList->removeItem(filesPage->FileList->currentItem() );
  filesPage->fileDelete->setEnabled( false );
  deleteFileFromSOURCE( Filename );
  saveProject = true;
}

void KRPMBuilder::slotDeleteResult( KIO::Job *job ) {
  KIO::DeleteJob *delJob = dynamic_cast<KIO::DeleteJob*> ( job );
  // just Display an error if it did not work.
  if ( delJob->error() != 0 ) {
    delJob->showErrorDialog( this );
  }
}

void KRPMBuilder::deleteFileFromSOURCE( QString & filename )
{
  KURL delURL = RPMsourceURL;
  delURL.addPath( filename );
  KIO::DeleteJob *delJob = KIO::del( delURL, false, true );
  connect( delJob, SIGNAL( result( KIO::Job* ) ), this, SLOT( slotDeleteResult( KIO::Job* ) ) );
}
  
/** No descriptions */
void KRPMBuilder::moveItemUp( KListBox *Listbox )
{
  int current = Listbox->currentItem();
  int moveTo = current - 1;
  QString  itemText = Listbox->currentText();
  Listbox->removeItem( current );
  Listbox->insertItem( itemText,moveTo );
  Listbox->setCurrentItem( moveTo );
  saveProject = true;
}
void KRPMBuilder::moveItemDown( KListBox *Listbox )
{
  int current = Listbox->currentItem();
  int moveTo = current + 1;
  QString  itemText = Listbox->currentText();
  Listbox->removeItem( current );
  Listbox->insertItem( itemText, moveTo );
  Listbox->setCurrentItem( moveTo );
  saveProject = true;
}

void KRPMBuilder::slotFileMoveUp(){
  moveItemUp( filesPage->FileList );
}
/** No descriptions */

void KRPMBuilder::slotFileMoveDown(){
  moveItemDown( filesPage->FileList );
}

/** No descriptions */
void KRPMBuilder::slotPatchAdd(){
  if ( filesPage->m_patch->text() != "" ) {
    filesPage->m_patch->clear();
    filesPage->patchAdd->setEnabled( false );
    targetWidget = filesPage->PatchList;
    copyFileToSOURCE( patchURL );
    saveProject = true;
  }
}
/** No descriptions */
void KRPMBuilder::slotPatchDelete(){
  QString Filename = KURL( filesPage->PatchList->currentText() ).fileName( true );
  filesPage->PatchList->removeItem( filesPage->PatchList->currentItem() );
  filesPage->patchDelete->setEnabled(false);
  deleteFileFromSOURCE( Filename );
  saveProject = true;
}
/** No descriptions */
void KRPMBuilder::slotPatchMoveUp() {
  moveItemUp( filesPage->PatchList );
}
/** No descriptions */
void KRPMBuilder::slotPatchMoveDown(){
  moveItemDown( filesPage->PatchList );
}

/** No descriptions */
void KRPMBuilder::slotSourceSearch(){
  KURL srcURL( KFileDialog::getOpenURL( QString::null,"*.tar.gz *.tar.bz2",this, i18n("Open...") ) );
  if ( ! srcURL.fileName( false ).isEmpty() ) {
    QString neededName = projectPage->m_name->text() + "-" + projectPage->m_version->text();
    std::cout << neededName << std::endl;
    QString selectedName = srcURL.fileName( false );
    std::cout << selectedName.mid( 0, neededName.length() ) << std::endl;
    if ( neededName == selectedName.mid( 0, neededName.length() ) ) {
      targetWidget = projectPage->m_file;
      sourceDir = KURL( srcURL.directory() );
      copyFileToSOURCE( srcURL );
      if ( ! copyError ) {
        filesPage->fileSearch->setEnabled( true );
        filesPage->patchSearch->setEnabled( true );
      }
    } else {
      KMessageBox::error( 0 , //No parent
        i18n("Wrong file selected for this name and version\nof the rpm to be built") , //Short!
        i18n("Bad select") ); //Caption
    }
  }
}

void KRPMBuilder::copyFileToSOURCE( KURL & fromURL )
{
  if ( fromURL.directory() != RPMsourceURL.path( -1 ) ) {
    // Copy file to RPM SOURCE directory if it is not already there.
    copyError = false;
    KURL destURL = RPMsourceURL;
    destURL.setFileName( fromURL.fileName( false ) );
    KIO::FileCopyJob *fileJob = KIO::file_copy( fromURL, destURL, -1, true, true, true );
    connect(fileJob, SIGNAL(result(KIO::Job*)), this, SLOT(slotCopyJobResult(KIO::Job*)));
  }
}

KURL KRPMBuilder::handleFileSearch( KListBox *ListBox, QString Filetypes ) {
  KListBox *alternateList;
  QLineEdit *alternateEdit;
  QString newFilename;
  
  if ( ListBox ==  filesPage->FileList ) {
    alternateList = filesPage->PatchList;
    alternateEdit = filesPage->m_patch;
  } else {
    alternateList = filesPage->FileList;
    alternateEdit = filesPage->m_file;
  } 
  KURL newURL( KFileDialog::getOpenURL( QString::null, Filetypes, ListBox, i18n("Open...") ) );
  newFilename = newURL.fileName( false );
  
  if ( newFilename == "" // nothing selected
       || ListBox->findItem( newFilename ) != 0   // already entered into this listbox
       || alternateList->findItem( newFilename ) != 0 // already entered into other list
       || alternateEdit->text() == newFilename // same filename is entered into the other tab-Page
       || projectPage->m_file->text() == newFilename ) { // filename is already entered as source into project-page
      newURL.setPath( "" );
  }
  return newURL;
}
/** No descriptions */
void KRPMBuilder::slotPatchSearch(){
  KURL srcURL;
  if ( (srcURL = handleFileSearch( filesPage->PatchList, "*" )).path( -1 ) != "" ) {
    // A new valid patch was selected
    filesPage->m_patch->setText( srcURL.fileName( false ) );
    patchURL = srcURL;
    filesPage->patchAdd->setEnabled( true );
  }
}
/** No descriptions */
void KRPMBuilder::slotFileSearch(){
  KURL srcURL;
  if ( (srcURL = handleFileSearch( filesPage->FileList, "*.tar.gz *.tar.bz2" )).path( -1 ) != "" ) {
    // A new valid source was selected
    filesPage->m_file->setText( srcURL.fileName( false ) );
    sourceURL = srcURL;
    filesPage->fileAdd->setEnabled( true );
  }
}
/** No descriptions */
void KRPMBuilder::slotCopyJobResult( KIO::Job *job ) {
  KIO::FileCopyJob *copyJob = dynamic_cast<KIO::FileCopyJob*> ( job );
  // If the copy job ended successfully, we move the name of the copied
  // file to the dialogfield / ListBox
  if ( copyJob->error() == 0 ) {
    // if we copied the files during startup, we need not display
    // or add them to a list.
    if ( targetWidget != 0 ) {
      if ( targetWidget == projectPage->m_file ) {
        ((QLineEdit *) targetWidget)->setText( copyJob->destURL().fileName( false ) );
      } else {
        ((KListBox *) targetWidget)->insertItem( copyJob->srcURL().path( -1 ) );
      }
    }
  } else {
    // in case of an error clear the field on the projectpage,
    // nothing gets done in the listboxes.
    if ( targetWidget == projectPage->m_file ) {
      ((QLineEdit *) targetWidget)->setText( "" );
    }
    copyJob->showErrorDialog( this );
    copyError = true;
  }
}

/** No descriptions */
void KRPMBuilder::slotSettings(){
  Settings setting;
  setting.pathPage->m_specs->setText( RPMspecURL.path( true ) );
  setting.pathPage->m_source->setText(RPMsourceURL.path( true ) );
  bool ok = setting.exec();
  if ( ok ) {
    RPMsourceURL.setPath(setting.pathPage->m_source->text() );
    RPMspecURL.setPath(setting.pathPage->m_specs->text() );
    hasPaths = true;
  }
}

/** No descriptions */
void KRPMBuilder::slotStopRPM() {
  proc->kill( SIGTERM );
}
void KRPMBuilder::slotStartupCopyResult( KIO::Job *job ) {
  KIO::CopyJob *copyJob = dynamic_cast<KIO::CopyJob*> ( job );
  // We are interested in errors only. Nothing gets done on sccess,
  // since everything should be in place already
  if ( copyJob->error() != 0 ) {
    copyJob->showErrorDialog( this );
    slotNew();
  }
}

void KRPMBuilder::copyAllFilesToSOURCE()
{
  targetWidget = 0;
  KURL srcURL = KURL( sourceDir.path( -1 ) + "/" + projectPage->m_file->text() );
  KURL::List FileList;
  // Construct a list of source files and patches to be copied to rpm SOURCE directory.
  FileList.append( srcURL );
  for ( uint i = 0; i < filesPage->FileList->count(); i++ ) {
    srcURL = KURL( filesPage->FileList->text( i ) );
    FileList.append( srcURL );
  }
  for ( uint i = 0; i < filesPage->PatchList->count(); i++ ) {
    srcURL = KURL( filesPage->PatchList->text( i ) );
    FileList.append( srcURL );
  }
  copyError = false;
  KIO::CopyJob *copyAll = KIO::copy( FileList, RPMsourceURL.path( -1 ), true );
  connect( copyAll, SIGNAL( result( KIO::Job* ) ), this, SLOT( slotStartupCopyResult( KIO::Job* ) ) );
}

bool KRPMBuilder::projectIsModified()
{
  // check projectPage for modifications
  if ( projectPage->m_name->isModified() )
    return true;
  if ( projectPage->m_version->isModified() )
    return true;
  if ( projectPage->m_packager->isModified() )
    return true;
  if ( projectPage->m_release->isModified() )
    return true;
   if ( projectPage->m_url->isModified() )
     return true;
   if ( projectPage->m_file->isModified() )
     return true;
  // The filesPage is not check for modifications here.
  // Each modification sets the global varibale saveProject to true.
  // check packagePage for modifications
  if( packagePage->m_distribution->isModified() )
    return true;
  if ( packagePage->m_description->isModified() )
    return true;
  if ( packagePage->m_provides->isModified() )
    return true;
  if ( packagePage->m_requires->isModified() )
    return true;
  if ( packagePage->m_summary->isModified() )
    return true;
  if ( packagePage->m_vendor->isModified() )
    return true;
  // check buildPage for modifications
  if ( buildPage->m_buildRoot->isModified() )
    return true;
  if ( buildPage->m_configure->isModified() )
    return true;
  if ( buildPage->m_prefix->isModified() )
    return true;
  // nothing changed, need not save the project file
  return false;
}
